/*****************************************************************************
 *****************************************************************************
 Copyright (c) 2000-2001, AEP Ltd. 

 All rights reserved.

 Redistribution and use in source and binary forms, with or without 
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.

 3. Neither the name of AEP Ltd. nor the names of its contributors 
 may be used to endorse or promote products derived from this software 
 without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

 *****************************************************************************
****************************************************************************/

/**********************************************************************
*                                                                     *
* AEP Ltd.                                                            *
*                                                                     *
* This software is supplied under the terms of the license included   *
* above.  All use of this software must be in accordance with the     *
* terms of that license.                                              *
*                                                                     *
**********************************************************************/

/* **************************************************************/
/*								*/
/* Filename:	trace.c						*/
/* Description:	This file contains the implementation of the 	*/
/*		trace functions. Tracing can be turned on or  	*/
/*		off with the AEP_TRACING_ON #define			*/
/*								*/
/* Copyright:	Accelerated Encryption Processing Ltd., 2000	*/
/* Author:	Diarmuid O'Neill				*/
/* Product:	AEP1000						*/
/* Date:	13/10/99					*/
/*								*/
/****************************************************************/

/*
	Trace.c
	Diarmuid O'Neill
	13-October-1999

	This file contains the implementation of the trace functions.
	Tracing can be turned on or off with the AEP_TRACING_ON #define

*/
#include "aep_i_defs.h"
#include "aep_i_types.h"
#include "trace.h"

#if defined(AEP_Win32)
#include "windows.h"
#include "winuser.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <assert.h>

/* Define this to turn on tracing */

#ifdef AEP_TRACING_ON

void AEP_I_LogEntry(
  AEP_CHAR_PTR FunctionName, /* Name of the current function */
  AEP_CHAR_PTR ProcessDesc,  /* Description of the current process */
  AEP_RV rv                 /* return value in case of imediate abort of function */
)
{
	FILE * log = NULL_PTR;
  
/*Open a different file depending on the os*/
#if defined(AEP_GENERIC) 
	log = fopen("aepdmn.log","a");
	if(log == NULL_PTR) return;
#elif defined(AEP_Win32)
	log = fopen("c:\\aepdmn.log","a");
	if(log == NULL_PTR)
    {
		MessageBox(NULL, "AEP_I_VarLogEntry: could not open Log-file", 
			"AEPAPI Notice", MB_OK|MB_ICONWARNING);
		return;
    }
#else
#error This platform is not supported
#endif  
	/*if fn = Null then fn = "(NULL)" else fn = fn*/
	FunctionName = (FunctionName == NULL_PTR)? (AEP_CHAR_PTR)"(NULL)" : FunctionName;
	/*dito*/
	ProcessDesc = (ProcessDesc == NULL_PTR)? (AEP_CHAR_PTR)"(NULL)" : ProcessDesc;

	fprintf(log,"** %s()", FunctionName);
	fprintf(log,": %s **\n", ProcessDesc);
  
	fclose(log);

	return;

}


void AEP_I_VarLogEntry(
  AEP_CHAR_PTR FunctionName,  /* Name of the current function */
  AEP_CHAR_PTR ProcessDesc,   /* Description of the current process */
  AEP_RV rv,                  /* return value in case of abort of function */
  ...
)
{
	FILE * log = NULL_PTR;
	va_list params;

	va_start (params,rv);

/*Open a different file depending on the os*/
#if defined(AEP_GENERIC) 
	log = fopen("aepdmn.log","a");
	if(log == NULL_PTR) {
		printf("Did not open file \n");
		return;
	}
#elif defined(AEP_Win32)
	log = fopen("c:\\aepdmn.log","a");
	if(log == NULL_PTR)
    {
		MessageBox(NULL, "AEP_I_VarLogEntry: could not open Log-file", 
			"AEPAPI Notice", MB_OK|MB_ICONWARNING);
		return;
    }
#else
#error This platform is not supported
#endif

  /*if fn = Null then fn = "(NULL)" else fn = fn*/
	FunctionName = (FunctionName == NULL_PTR)? "(NULL)" : FunctionName;
  /*dito*/
	ProcessDesc = (ProcessDesc == NULL_PTR)? "(NULL)" : ProcessDesc;

	fprintf(log,"** %s(): ", FunctionName);

	vfprintf(log, ProcessDesc, params);
	fprintf(log,"**\n");
	va_end(params);

	fclose(log);

	return;
}


#else /* ! AEP_TRACING_ON */

void AEP_I_LogEntry(
  AEP_CHAR_PTR FunctionName, /* Name of the current function */
  AEP_CHAR_PTR ProcessDesc,  /* Description of the current process */
  AEP_RV rv                 /* return value in case of imediate abort of function */
)
{
	return;
}

void AEP_I_VarLogEntry(
  AEP_CHAR_PTR FunctionName, /* Name of the current function */
  AEP_CHAR_PTR ProcessDesc,  /* Description of the current process */
  AEP_RV rv,                 /* return value in case of abort of function */
  ...
  )
{
 return;
}

#endif /* ! AEP_TRACING_ON */
