/*****************************************************************************
 *****************************************************************************
 Copyright (c) 2000-2001, AEP Ltd. 

 All rights reserved.

 Redistribution and use in source and binary forms, with or without 
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.

 3. Neither the name of AEP Ltd. nor the names of its contributors 
 may be used to endorse or promote products derived from this software 
 without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

 *****************************************************************************
****************************************************************************/

/**********************************************************************
*                                                                     *
* AEP Ltd.                                                            *
*                                                                     *
* This software is supplied under the terms of the license included   *
* above.  All use of this software must be in accordance with the     *
* terms of that license.                                              *
*                                                                     *
**********************************************************************/

/*Definitions and Function Prototypes*/
#ifndef HEADER_DEFINITIONS_H
#define HEADER_DEFINITIONS_H

#ifndef WIN16
#include <stdio.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

/* an unsigned 8-bit value */
typedef unsigned char  BYTE;
#define BN_MUL_COMBA
#define BN_SQR_COMBA
#define BN_RECURSION
#define RECP_MUL_MOD
#define MONT_MUL_MOD

/*#ifdef WIN32*/

#define BN_ULLONG unsigned long long
#endif

#define BN_ULONG unsigned long
#define BN_LONG long
#define BN_BITS 64
#define BN_BYTES 4
#define BN_BITS2 32
#define BN_BITS4 16
#ifdef WIN32
#define BN_MASK				(0xffffffffffffffffL)
#else
#define BN_MASK				(0xffffffffffffffffLL)
#endif
#define BN_MASK2			(0xffffffffuL) 
#define BN_MASK2l			(0xffff)
#define BN_MASK2h1			(0xffff8000L)
#define BN_MASK2h			(0xffff0000L)
#define BN_TBIT				(0x80000000L)
#define BN_DEC_CONV			(1000000000L)
#define BN_DEC_FMT1			"%lu"
#define BN_DEC_FMT2			"%09lu"
#define CN_DEC_NUM			9

#ifdef BIGNUM
#undef BIGNUM
#endif

#define BN_FLAG_MALLOCED	0x01
#define BN_FLG_STATIC_DATA	0x02
#define BN_FLG_FREE		0x8000	/* used for debuging */
#define BN_set_flags(b,n)	((b)->flags|=(n))
#define BN_get_flags(b,n)	((b)->flags&(n))


#define	L2HBITS(a)	((BN_ULONG)((a)&BN_MASK2l)<<BN_BITS4)

typedef struct bignum_st
	{
	BN_ULONG *d;	/* Pointer to an array of 'BN_BITS2' bit chunks. */
	int top;	/* Index of last used d +1. */
	/* The next are internal book keeping for bn_expand. */
	int max;	/* Size of the d array. */
	int neg;	/* one if the number is negative */
	int flags;
	} BIGNUM;


/*	Used for temp variables	*/
#define BN_CTX_NUM			19

#define BN_FLG_MALLOCED		0x01
typedef struct bignum_ctx
{
	int tos;
	int flags;
	BIGNUM bn[BN_CTX_NUM+1];/*Array of 20 big numbers						*/
}	BN_CTX;


#if defined (SA)
#define Malloc		(char*(*)()) SAr_malloc
#define Free(addr)	SAr_free(addr)

#else

#define Malloc		(char*(*)()) malloc
#define Free(addr)	free(addr)
#endif


#define BN_is_word(a,w)	(((a)->top == 1) && ((a)->d[0] == (BN_ULONG)(w)))
#define BN_is_one(a)	(BN_is_word((a),1));
#define bn_wexpand(n,b)		(((b) <= (n)->max)?(n):bn_expand2((n),(b)))
#define bn_wexpand2(n,b)		(((b) <= (n).max)?(n):bn_expandCTX((n),(b)))
#define BN_zero(a)	(BN_set_word((a),0))
#define bn_expand(n,b) ((((((b+BN_BITS2-1))/BN_BITS2)) <= (n)->max)?\
	(n):bn_expand2((n),(b)/BN_BITS2+1))

#define Lw(t)    (((BN_ULONG)(t))&BN_MASK2)
#define Hw(t)    (((BN_ULONG)((t)>>BN_BITS2))&BN_MASK2)

/* These are used for internal error checking and are not normally used */
#ifdef BN_DEBUG
#define bn_check_top(a) \
	{ if (((a)->top < 0) || ((a)->top > (a)->max)) \
		{ char *nullp=NULL; *nullp='z'; } }
#define bn_check_num(a) if ((a) < 0) { char *nullp=NULL; *nullp='z'; }
#else
#define bn_check_top(a)
#define bn_check_num(a)
#endif

#define BN_MULL_SIZE_NORMAL			(16) /* 32 */
#define BN_MUL_RECURSIVE_SIZE_NORMAL		(16) /* 32 less than */
#define BN_SQR_RECURSIVE_SIZE_NORMAL		(16) /* 32 */
#define BN_MUL_LOW_RECURSIVE_SIZE_NORMAL	(32) /* 32 */
#define BN_MONT_CTX_SET_SIZE_WORD		(64) /* 32 */


#define LBITS(a)	((a)&BN_MASK2l)
#define HBITS(a)	(((a)>>BN_BITS4)&BN_MASK2l)

#define bn_fix_top(a) \
        { \
        BN_ULONG *ftl; \
	if ((a)->top > 0) \
		{ \
		for (ftl= &((a)->d[(a)->top-1]); (a)->top > 0; (a)->top--) \
		if (*(ftl--)) break; \
		} \
	}


#define mul(r,a,w,c) { \
	BN_ULLONG t; \
	t=(BN_ULLONG)w * (a) + (c); \
	(r)= Lw(t); \
	(c)= Hw(t); \
	}
#define sqr(r, a, c) {\
	BN_ULLONG t; \
	t = (BN_ULLONG) (a) * (a) + (r);\
	(r) = Lw(t);\
    (c) = Hw(t);\
	}
			

#define mul_add(r,a,w,c) { \
	BN_ULLONG t; \
	t=(BN_ULLONG)w * (a) + (r) + (c); \
	(r)= Lw(t); \
	(c)= Hw(t); \
	}


#define mul64(l,h,bl,bh) \
	{ \
	BN_ULONG m,m1,lt,ht; \
 \
	lt=l; \
	ht=h; \
	m =(bh)*(lt); \
	lt=(bl)*(lt); \
	m1=(bl)*(ht); \
	ht =(bh)*(ht); \
	m=(m+m1)&BN_MASK2; if (m < m1) ht+=L2HBITS(1L); \
	ht+=HBITS(m); \
	m1=L2HBITS(m); \
	lt=(lt+m1)&BN_MASK2; if (lt < m1) ht++; \
	(l)=lt; \
	(h)=ht; \
	}

#define BN_is_zero(a)	(((a)->top == 0) || BN_is_word(a,0))
#define BN_is_word(a,w)	(((a)->top == 1) && ((a)->d[0] == (BN_ULONG)(w)))
#define BN_is_odd(a)	(((a)->top > 0) && ((a)->d[0] & 1))
#define BN_one(a)	(BN_set_word((a),1))

#define sqr64(lo,ho,in) \
	{ \
	BN_ULONG l,h,m; \
 \
	h=(in); \
	l=LBITS(h); \
	h=HBITS(h); \
	m =(l)*(h); \
	l*=l; \
	h*=h; \
	h+=(m&BN_MASK2h1)>>(BN_BITS4-1); \
	m =(m&BN_MASK2l)<<(BN_BITS4+1); \
	l=(l+m)&BN_MASK2; if (l < m) h++; \
	(lo)=l; \
	(ho)=h; \
	}


/* Function Prototypes */
/*Big Number */

void BN_CTX_free(BN_CTX *c);
void BN_CTX_init(BN_CTX *ctx);
BN_CTX *BN_CTX_new(void);
BIGNUM *BN_new(void);
void BN_init(BIGNUM *);
/*void PrintBigNum(BIGNUM *a); */
BIGNUM* BN_SetDiff(void);
BIGNUM* BN_Set(void);
void BN_clear_free(BIGNUM *a);
int BN_set_word(BIGNUM *a, BN_ULONG w);
BIGNUM *bn_expand2(BIGNUM *b, int words);
/*Addition*/
int BN_add(BIGNUM *r, BIGNUM *a, BIGNUM *b);
int BN_uadd(BIGNUM *r, BIGNUM *a, BIGNUM *b);
BN_ULONG bn_add_words(BN_ULONG *r, BN_ULONG *a, BN_ULONG *b, int n);
/*Subtraction*/
int BN_sub(BIGNUM *r, BIGNUM *a, BIGNUM *b);
int BN_usub(BIGNUM *r, BIGNUM *a, BIGNUM *b);
BN_ULONG bn_sub_words(BN_ULONG *r, BN_ULONG *a, BN_ULONG *b, int n);
int multiply(BIGNUM *r, BIGNUM *a, BIGNUM *b, BIGNUM *tempA, BIGNUM *tempB);
BIGNUM *BN_copy(BIGNUM *a, BIGNUM *b);
int BN_rshift(BIGNUM *r, BIGNUM *a, int n);
int BN_ucmp(BIGNUM *a, BIGNUM *b);
int BN_sub(BIGNUM *r, BIGNUM *a, BIGNUM *b);
int countBits(BIGNUM * p);
BIGNUM *shiftLeft(BIGNUM *b);
void BN_CBytesToBigNum(BIGNUM *a, BYTE* number, int length);
int countExactNumberOfBits(BIGNUM *a);
int countExactNumberOfBitsInWord(BN_ULONG a);
BN_ULONG bn_mul_words(BN_ULONG *rp, BN_ULONG *ap, int num, BN_ULONG w);
int BN_div(BIGNUM *dv, BIGNUM *rm, BIGNUM *num, BIGNUM *divisor, BN_CTX *ctx);
BN_ULONG bn_div_words(BN_ULONG h, BN_ULONG l, BN_ULONG d);
int BN_lshift(BIGNUM *r, BIGNUM *a, int n);
int extendedGCD(BIGNUM* GCD, BIGNUM *modInverse, BIGNUM *nPrime, BIGNUM *x, BIGNUM *y, BN_CTX *ctx);
int MonPro(BIGNUM *result, BIGNUM *a, BIGNUM *b,  BIGNUM *modulus,  BIGNUM* rInverseModN, BN_CTX* monProCTX,  BN_CTX* divisionCTX);
BYTE* bigNumberToBytes(BIGNUM *a, int *length);
void BN_SetNumberValue(BIGNUM *a, BYTE* number, int length, int noOfPositions);
BIGNUM *bytesToBigNumber(BYTE* number, int length, int arrayLength);
int testGCDResults(BIGNUM* R, BIGNUM *rInverse, BIGNUM *n, BIGNUM *nPrime, BN_CTX* temp);
int modExp(BIGNUM * result, BIGNUM *a, BIGNUM *e, BIGNUM *n);
int mult(BIGNUM *result, BIGNUM *a, BIGNUM *b);
int BN_mod(BIGNUM *rem, BIGNUM *m, BIGNUM *d, BN_CTX *ctx);
int expMinusOne(BIGNUM* result, BIGNUM *exp);
int mPlusOne(BIGNUM *result, BIGNUM *a);
int mPlusOneOverTwo(BIGNUM *result, BIGNUM *m);
void *CRYPTO_malloc(int num);
void CRYPTO_free(void *str);
int square(BIGNUM *result, BIGNUM *a);
BN_ULONG bn_mul_add_words(BN_ULONG *rp, BN_ULONG *ap, int num, BN_ULONG w);
BN_ULONG bn_mul_words(BN_ULONG *rp, BN_ULONG *ap, int num, BN_ULONG w);
void bn_sqr(BN_ULONG *r, BN_ULONG *a, int n, BN_ULONG *tmp);

int rSquaredModN(BIGNUM *result, BIGNUM *r, BIGNUM *n, BN_CTX* ctx);
int calcResidue(BIGNUM *result, BIGNUM* a, BIGNUM* RModN, BIGNUM* n,  BN_CTX* ctx);
int calcR(BIGNUM* R, int noOfBitsInModulus);
int calcRModN(BIGNUM* RModN, BIGNUM* R, BIGNUM *n, BN_CTX* divisionCTX);


BIGNUM* BN_SetBase();
BIGNUM* BN_SetPower();
BIGNUM* BN_SetMod();

BIGNUM* BN_SetCorrectnessBase();
BIGNUM* BN_SetCorrectnessPower();
BIGNUM* BN_SetCorrectnessMod();
BIGNUM* BN_SetCorrectnessP();
BIGNUM* BN_SetCorrectnessQ();
BIGNUM* BN_SetCorrectnessDmpi();
BIGNUM* BN_SetCorrectnessDmqi();
BIGNUM* BN_SetCorrectnessIqmp();
BIGNUM* BN_SetCorrectnessResult();

